/* 
 * wsdlpull - A C++ parser  for WSDL  (Web services description language)
 * Copyright (C) 2005-2007 Vivek Krishna
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


//A generic web service invocation tool which uses the invocation API
#include "wsdlparser/WsdlInvoker.h"
using namespace std;
using namespace WsdlPull;

void
usage(void)
{
  std::cout<<"Usage wsdl [options] wsdl-uri [operation name] [method parameters]"<<std::endl;
  std::cout<<"Options: "<<std::endl;
  std::cout<<"   -h  Display this message"<<std::endl;
  std::cout<<"   -x host[:port] Use HTTP proxy on given port"<<std::endl;
  std::cout<<"   -U user[:password] Specify Proxy authentication"<<std::endl;
  std::cout<<"   -v Verbose mode,SOAP request and response are logged"<<std::endl;
  std::cout<<"   -d display WSDL operation's documentation"<<std::endl;
  std::cout<<"   -l list all the WSDL operations "<<std::endl;
  std::cout<<"   -o Allow setting occurrence constraint (default is 1)"<<std::endl;
  std::cout<<"   -s Suppress printing type/element names in the output"<<std::endl;
  std::cout<<"   -t requesttimeout in seconds"<<std::endl;
  std::cout<<"   -e set SOAP headers in input"<<std::endl;
  std::cout<<"With no arguments,wsdl starts in the interactive mode accepting"<<std::endl;  
  std::cout<<"operation name and parameters from the standard input"<<std::endl;
}

int 
main (int argc, char *argv[]) 
{
  WsdlInvoker invoker;
  bool brkloop =false;
  bool showDoc = false;
  bool verbose = false;
  bool occurs = false;
  bool listops = false;
  bool accept_password =false;
  bool accept_headers = false;
  long timeout = 0;

  int i =1;
  for (;i<argc && !brkloop;){
    switch(argv[i][0]){
    case '-'://option
      {
	std::string opt(argv[i]+1);
	if (opt=="v"){
	  invoker.setVerbose(true);
	  verbose = true;
	  showDoc = true;
	  i++;
	}
	else if (opt == "s"){
	  
	  invoker.printTypeNames(false);
	  i++;
	}
	else if (opt == "d"){
	  
	  showDoc = true;
	  i++;
	}
	else if (opt == "e"){
	  
	  accept_headers = true;
	  i++;
	}
	else if (opt == "l"){
	  
	  listops=true;
	  i++;
	}
	else if (opt == "x"){
	  opt = argv[i+1];
	  size_t pos=opt.find(':');
          XmlUtils::setProxyHost (opt);
	  if(pos==std::string::npos){
	    
            XmlUtils::setProxyHost (XmlUtils::getProxyHost () + ":80");
          }
          XmlUtils::setProxy (true);
	  i+=2;
	}
	else if (opt == "U"){
	  opt = argv[i+1];
	  size_t pos=opt.find(':');
          XmlUtils::setProxyUser (opt.substr(0,pos));
	  if(pos!=std::string::npos)
            XmlUtils::setProxyPass (opt.substr(pos+1));
	  else
	    accept_password = true;
	  i+=2;
          XmlUtils::setProxy (true);
	}
	else if (opt =="h"){
	  usage();
	  exit(0);
	}
	else if(opt == "o"){
	  
	  occurs = true;//ask for occurrence constraints
	  i++;
	}
	else if(opt == "t"){
	  opt = argv[i+1];
	  timeout=atoi(opt.c_str());
	  i+=2;
	}
	else{
	  std::cerr<<"Unknown option "<<argv[i]<<std::endl;
	  usage();
	  exit(2);
	}
	break;
      }
    default:
      brkloop = true;
      //end of options
      break;
    }
  }

  if (XmlUtils::getProxy () && accept_password){
     
    XmlUtils::setProxyPass (XmlUtils::acceptSecretKey("Proxy Password"));
    std::cout<<endl;
  }

  if (i < argc){
    if(!invoker.setWSDLUri(argv[i])) {

      std::cerr<<"Error processing "<<argv[i]<<std::endl;
      std::cerr<<invoker.errors()<<std::endl;
      return 1;
    }
    i++;
  }
  else{
    
    usage();
    exit (2);
  }

  if (verbose)
    std::cout<<invoker.errors()<<std::endl;
  
  if (i<argc && !listops){
    
    if(!invoker.setOperation(argv[i])){
      
      std::cerr<<"Unkown operation name "<<argv[i]<<std::endl;
      return 2;
    }
    i++;
  }
  else{
  
    std::vector<std::string> ops;
    unsigned int choice = 0;
    if (invoker.getOperations(ops)){
    
      for (size_t s = 0;s<ops.size();s++){
      
	std::cout<<s+1<<"."<<ops[s];
	
	if (showDoc) {
	 
	  std::string doc = invoker.getOpDocumentaion(ops[s]);
	  if (!doc.empty())
	    std::cout<<"("<<doc<<")";
	}
	std::cout<<endl;
      }
      if (listops == true){
	
	return 0;
      }
      while (choice==0){
	
	cout<<"Choose one of the above operations [1-"<<ops.size()<<"] :";
	std::cin>>choice;
	if (choice>0 && choice<=ops.size())
	  break;
	else
	  choice=0;
      }
    }
    if (!invoker.setOperation(ops[choice-1])){
      
      std::cerr<<"Couldn't invoke operation "<<std::endl;
      return 1;
    }
  }
  if(!accept_headers && invoker.nInputHeaders()>0){
    
    std::cout<<"Warning:This operation has some SOAP headers in its inputs!(use -e)"<<std::endl;
  }

  if (invoker.status()){
  
    int id =0,minimum,maximum,n;
    Schema::Type t;
    std::string param;
    std::string val;
    std::vector<std::string> values;
    std::vector<std::string> parents;
    
    do{    

      if (accept_headers && invoker.nInputHeaders()>0){
	
        id = invoker.getNextHeaderInput(param,t,minimum,maximum,parents);
	if (id == -1){
	  accept_headers=false;//done with headers
	  continue;
	}
      }
      else{
	
        id = invoker.getNextInput(param,t,minimum,maximum,parents);
      }
      if (id == -1)
	break;
      n = minimum;
      if (occurs && minimum < maximum) {
	values.clear();
        cout<<param<<"["<<minimum<<","<<maximum<<"] Enter number of occurrences:";
	cin>>n;
	
        if (n<minimum || n>maximum){
	  
	  std::cerr<<"Didnt match occurrence constraints"<<std::endl;
	  return 2;
	}
	while(n--) {

	  if (i <argc) {
	    val = argv[i++];
	  }
	  else {
	    cout<<param<<": ";
	    cin>>val;
	  }
	  values.push_back(val);
	}
	if (!invoker.setInputValue(id,values)){

	  std::cerr<<"Incorrect input values "<<std::endl;
	  return 2;
	}
      }
      else{

	if (i <argc) {
	  
	  val = argv[i++];
	}
	else{
	  size_t j = 0;
	  for (j=0;j<parents.size()-1;j++){

	    cout<<parents[j]<<".";
	  }
	  cout<<parents[j]<<": ";
	  cin>>val;
	}
	if (!invoker.setInputValue(id,val)){

	  std::cerr<<"Incorrect input value "<<val<<std::endl;
	  return 2;
	}
      }
    }while(1);
  
    if (invoker.invoke(timeout)){

      TypeContainer* tc = 0;
      std::string name;
      while(invoker.getNextHeaderOutput(name,tc)) {
        
        tc->print(cout);
      }
      
      while (invoker.getNextOutput(name,tc)){
	    
	tc->print(cout);
      }
      return 0;
    }
    else{
      cerr<<invoker.errors()<<endl;
    }
  }
  return 1;
}


